package net.deliciousreya.minecraftportal.model

import net.deliciousreya.minecraftportal.proto.PortalSaveDataProtos
import org.bukkit.Material
import java.lang.IllegalArgumentException
import java.lang.IllegalStateException

class PortalType(val mineral: Material) {
    var oldestActivePortal: Portal? = null
    var newestActivePortal: Portal? = null

    val isEmpty: Boolean get() = oldestActivePortal == null && newestActivePortal == null
    val isPaired: Boolean get() = oldestActivePortal != null && newestActivePortal != null

    val getOnlyPortal: Portal get() = if (!isEmpty && !isPaired) oldestActivePortal!! else throw IllegalStateException("Can't getOnlyPortal for empty or paired portal type $this")

    fun toProto(): PortalSaveDataProtos.PortalPair {
        val builder = PortalSaveDataProtos.PortalPair.newBuilder()
        val oldestPortal = oldestActivePortal
        val newestPortal = newestActivePortal
        if (oldestPortal != null) {
            builder.older = oldestPortal.toProto()
        }
        if (newestPortal != null) {
            builder.newer = newestPortal.toProto()
        }
        return builder.build()
    }

    fun addOrReplacePortal(portal: Portal): Portal? {
        if (portal.type != this) {
            throw DeserializationException("Tried to add $portal which didn't belong to ${this}")
        }
        return when {
            this.isEmpty  -> {
                this.oldestActivePortal = portal
                null
            }
            else -> {
                val replacedPortal = newestActivePortal
                this.newestActivePortal = portal
                replacedPortal
            }
        }
    }

    fun loadUnpairedPortal(portal: Portal) {
        if (!this.isEmpty) {
            throw DeserializationException("Already have a value for ${this} when adding $portal")
        }
        if (portal.type != this) {
            throw DeserializationException("Tried to add $portal which didn't belong to ${this}")
        }
        this.oldestActivePortal = portal
    }

    fun loadPairedPortals(olderPortal: Portal, newerPortal: Portal) {
        if (!this.isEmpty) {
            throw DeserializationException("Already have a value for ${this} when adding $olderPortal and $newerPortal")
        }
        if (olderPortal.type != this) {
            throw DeserializationException("Tried to add $olderPortal which didn't belong to ${this}")
        }
        if (newerPortal.type != this) {
            throw DeserializationException("Tried to add $newerPortal which didn't belong to ${this}")
        }
        this.oldestActivePortal = olderPortal
        this.newestActivePortal = newerPortal
    }

    fun clear() {
        oldestActivePortal = null
        newestActivePortal = null
    }

    override fun toString():String {
        return "PortalType{mineral=$mineral, oldestActivePortal=$oldestActivePortal, newestActivePortal=$newestActivePortal}"
    }

    fun removePortalWithFrame(frame: PortalFrame):Boolean {
        if (oldestActivePortal?.frame == frame) {
            oldestActivePortal = newestActivePortal
        } else if (newestActivePortal?.frame != frame) {
            return false
        }
        newestActivePortal = null
        return true
    }
}